@@ -96,15 +96,15 @@ class LogChunkEndpoint(LogChunkEndpointBase):
     # offset/limit query params in ResultSpec
     kind = base.EndpointKind.SINGLE
     isPseudoCollection = True
-    pathPatterns = """
-        /logchunks
-        /logs/n:logid/contents
-        /steps/n:stepid/logs/i:log_slug/contents
-        /builds/n:buildid/steps/i:step_name/logs/i:log_slug/contents
-        /builds/n:buildid/steps/n:step_number/logs/i:log_slug/contents
-        /builders/n:builderid/builds/n:build_number/steps/i:step_name/logs/i:log_slug/contents
-        /builders/n:builderid/builds/n:build_number/steps/n:step_number/logs/i:log_slug/contents
-    """
+    pathPatterns = [
+        "/logchunks",
+        "/logs/n:logid/contents",
+        "/steps/n:stepid/logs/i:log_slug/contents",
+        "/builds/n:buildid/steps/i:step_name/logs/i:log_slug/contents",
+        "/builds/n:buildid/steps/n:step_number/logs/i:log_slug/contents",
+        "/builders/n:builderid/builds/n:build_number/steps/i:step_name/logs/i:log_slug/contents",
+        "/builders/n:builderid/builds/n:build_number/steps/n:step_number/logs/i:log_slug/contents",
+    ]
     rootLinkName = "logchunks"
 
     @defer.inlineCallbacks
@@ -132,24 +132,19 @@ class LogChunkEndpoint(LogChunkEndpointBase):
         logLines = yield self.master.db.logs.getLogLines(logid, firstline, lastline)
         return {'logid': logid, 'firstline': firstline, 'content': logLines}
 
-    def get_kwargs_from_graphql(self, parent, resolve_info, args):
-        if parent is not None:
-            return self.get_kwargs_from_graphql_parent(parent, resolve_info.parent_type.name)
-        return {"logid": args["logid"]}
-
 
 class RawLogChunkEndpoint(LogChunkEndpointBase):
     # Note that this is a singular endpoint, even though it overrides the
     # offset/limit query params in ResultSpec
     kind = base.EndpointKind.RAW
-    pathPatterns = """
-        /logs/n:logid/raw
-        /steps/n:stepid/logs/i:log_slug/raw
-        /builds/n:buildid/steps/i:step_name/logs/i:log_slug/raw
-        /builds/n:buildid/steps/n:step_number/logs/i:log_slug/raw
-        /builders/n:builderid/builds/n:build_number/steps/i:step_name/logs/i:log_slug/raw
-        /builders/n:builderid/builds/n:build_number/steps/n:step_number/logs/i:log_slug/raw
-    """
+    pathPatterns = [
+        "/logs/n:logid/raw",
+        "/steps/n:stepid/logs/i:log_slug/raw",
+        "/builds/n:buildid/steps/i:step_name/logs/i:log_slug/raw",
+        "/builds/n:buildid/steps/n:step_number/logs/i:log_slug/raw",
+        "/builders/n:builderid/builds/n:build_number/steps/i:step_name/logs/i:log_slug/raw",
+        "/builders/n:builderid/builds/n:build_number/steps/n:step_number/logs/i:log_slug/raw",
+    ]
 
     @defer.inlineCallbacks
     def get(self, resultSpec, kwargs):
@@ -179,14 +174,14 @@ class RawInlineLogChunkEndpoint(LogChunkEndpointBase):
     # Note that this is a singular endpoint, even though it overrides the
     # offset/limit query params in ResultSpec
     kind = base.EndpointKind.RAW_INLINE
-    pathPatterns = """
-        /logs/n:logid/raw_inline
-        /steps/n:stepid/logs/i:log_slug/raw_inline
-        /builds/n:buildid/steps/i:step_name/logs/i:log_slug/raw_inline
-        /builds/n:buildid/steps/n:step_number/logs/i:log_slug/raw_inline
-        /builders/n:builderid/builds/n:build_number/steps/i:step_name/logs/i:log_slug/raw_inline
-        /builders/n:builderid/builds/n:build_number/steps/n:step_number/logs/i:log_slug/raw_inline
-    """
+    pathPatterns = [
+        "/logs/n:logid/raw_inline",
+        "/steps/n:stepid/logs/i:log_slug/raw_inline",
+        "/builds/n:buildid/steps/i:step_name/logs/i:log_slug/raw_inline",
+        "/builds/n:buildid/steps/n:step_number/logs/i:log_slug/raw_inline",
+        "/builders/n:builderid/builds/n:build_number/steps/i:step_name/logs/i:log_slug/raw_inline",
+        "/builders/n:builderid/builds/n:build_number/steps/n:step_number/logs/i:log_slug/raw_inline",
+    ]
 
     @defer.inlineCallbacks
     def get(self, resultSpec, kwargs):
@@ -217,11 +212,10 @@ class LogChunk(base.ResourceType):
     name = "logchunk"
     plural = "logchunks"
     endpoints = [LogChunkEndpoint, RawLogChunkEndpoint, RawInlineLogChunkEndpoint]
-    keyField = "logid"
 
     class EntityType(types.Entity):
         logid = types.Integer()
         firstline = types.Integer()
         content = types.String()
 
-    entityType = EntityType(name, 'LogChunk')
+    entityType = EntityType(name)
