@@ -13,11 +13,40 @@
 #
 # Copyright Buildbot Team Members
 
+from __future__ import annotations
+
+import dataclasses
+from typing import TYPE_CHECKING
+
 import sqlalchemy as sa
 from twisted.python import log
 
 from buildbot.data import base
 
+if TYPE_CHECKING:
+    from typing import Sequence
+
+
+class NotSupportedFieldTypeError(TypeError):
+    def __init__(self, data, *args: object) -> None:
+        super().__init__(
+            (f"Unsupported data type '{type(data)}': must be an instance of Dict or a Dataclass."),
+            *args,
+        )
+
+
+def _data_getter(d, fld):
+    if isinstance(d, dict):
+        return d[fld]
+    if dataclasses.is_dataclass(d):
+        try:
+            return getattr(d, fld)
+        except AttributeError as e:
+            # backward compatibility when only dict was allowed
+            raise KeyError(*e.args) from e
+
+    raise NotSupportedFieldTypeError(d)
+
 
 class FieldBase:
     """
@@ -71,10 +100,12 @@ class FieldBase:
         # only support string values, because currently there are no queries against lists in SQL
     }
 
-    def __init__(self, field, op, values):
+    def __init__(self, field: bytes | str, op: str, values: Sequence | set):
         self.field = field
         self.op = op
         self.values = values
+        # `str` is a Sequence as well...
+        assert not isinstance(values, str)
 
     def getOperator(self, sqlMode=False):
         v = self.values
@@ -95,7 +126,7 @@ class FieldBase:
         fld = self.field
         v = self.values
         f = self.getOperator()
-        return (d for d in data if f(d[fld], v))
+        return (d for d in data if f(_data_getter(d, fld), v))
 
     def __repr__(self):
         return f"resultspec.{self.__class__.__name__}('{self.field}','{self.op}',{self.values})"
@@ -184,7 +215,7 @@ class ReverseComparator:
 
 
 class ResultSpec:
-    __slots__ = ['filters', 'fields', 'properties', 'order', 'limit', 'offset', 'fieldMapping']
+    __slots__ = ['fieldMapping', 'fields', 'filters', 'limit', 'offset', 'order', 'properties']
 
     def __init__(
         self, filters=None, fields=None, properties=None, order=None, limit=None, offset=None
@@ -372,13 +403,18 @@ class ResultSpec:
             fields = set(self.fields)
 
             def includeFields(d):
-                return dict((k, v) for k, v in d.items() if k in fields)
+                if isinstance(d, dict):
+                    return dict((k, v) for k, v in d.items() if k in fields)
+                elif dataclasses.is_dataclass(d):
+                    raise TypeError("includeFields can't filter fields of dataclasses")
+                raise NotSupportedFieldTypeError(d)
 
             applyFields = includeFields
         else:
             fields = None
+            applyFields = None
 
-        if isinstance(data, dict):
+        if isinstance(data, dict) or dataclasses.is_dataclass(data):
             # item details
             if fields:
                 data = applyFields(data)
@@ -390,9 +426,9 @@ class ResultSpec:
             # item collection
             if isinstance(data, base.ListResult):
                 # if pagination was applied, then fields, etc. must be empty
-                assert (
-                    not fields and not order and not filters
-                ), "endpoint must apply fields, order, and filters if it performs pagination"
+                assert not fields and not order and not filters, (
+                    "endpoint must apply fields, order, and filters if it performs pagination"
+                )
                 offset = data.offset
                 total = data.total
                 limit = data.limit
@@ -419,8 +455,7 @@ class ResultSpec:
                     Do a multi-level sort by passing in the keys
                     to sort by.
 
-                    @param elem: each item in the list to sort.  It must be
-                              a C{dict}
+                    @param elem: each item in the list to sort.
                     @param order: a list of keys to sort by, such as:
                                 ('lastName', 'firstName', 'age')
                     @return: a key used by sorted(). This will be a
@@ -436,7 +471,7 @@ class ResultSpec:
                             # it means sort by 'lastName' in reverse.
                             k = k[1:]
                             doReverse = True
-                        val = NoneComparator(elem[k])
+                        val = NoneComparator(_data_getter(elem, k))
                         if doReverse:
                             val = ReverseComparator(val)
                         compareKey.append(val)
