@@ -16,6 +16,7 @@
 from __future__ import annotations
 
 from typing import TYPE_CHECKING
+from typing import Any
 
 from twisted.internet import defer
 
@@ -24,27 +25,27 @@ from buildbot.data import types
 
 if TYPE_CHECKING:
     from buildbot.db.test_results import TestResultModel
+    from buildbot.util.twisted import InlineCallbacksType
 
 
-class Db2DataMixin:
-    def db2data(self, model: TestResultModel):
-        return {
-            'test_resultid': model.id,
-            'builderid': model.builderid,
-            'test_result_setid': model.test_result_setid,
-            'test_name': model.test_name,
-            'test_code_path': model.test_code_path,
-            'line': model.line,
-            'duration_ns': model.duration_ns,
-            'value': model.value,
-        }
+def _db2data(model: TestResultModel):
+    return {
+        'test_resultid': model.id,
+        'builderid': model.builderid,
+        'test_result_setid': model.test_result_setid,
+        'test_name': model.test_name,
+        'test_code_path': model.test_code_path,
+        'line': model.line,
+        'duration_ns': model.duration_ns,
+        'value': model.value,
+    }
 
 
-class TestResultsEndpoint(Db2DataMixin, base.Endpoint):
+class TestResultsEndpoint(base.Endpoint):
     kind = base.EndpointKind.COLLECTION
-    pathPatterns = """
-        /test_result_sets/n:test_result_setid/results
-        """
+    pathPatterns = [
+        "/test_result_sets/n:test_result_setid/results",
+    ]
 
     @defer.inlineCallbacks
     def get(self, resultSpec, kwargs):
@@ -59,17 +60,16 @@ class TestResultsEndpoint(Db2DataMixin, base.Endpoint):
             set_dbdict.builderid, kwargs['test_result_setid'], result_spec=resultSpec
         )
 
-        return [self.db2data(result) for result in result_dbdicts]
+        return [_db2data(result) for result in result_dbdicts]
 
 
 class TestResult(base.ResourceType):
     name = "test_result"
     plural = "test_results"
     endpoints = [TestResultsEndpoint]
-    keyField = 'test_resultid'
-    eventPathPatterns = """
-        /test_result_sets/:test_result_setid/results
-    """
+    eventPathPatterns = [
+        "/test_result_sets/:test_result_setid/results",
+    ]
 
     class EntityType(types.Entity):
         test_resultid = types.Integer()
@@ -81,11 +81,13 @@ class TestResult(base.ResourceType):
         duration_ns = types.NoneOk(types.Integer())
         value = types.String()
 
-    entityType = EntityType(name, 'TestResult')
+    entityType = EntityType(name)
 
     @base.updateMethod
     @defer.inlineCallbacks
-    def addTestResults(self, builderid, test_result_setid, result_values):
+    def addTestResults(
+        self, builderid: int, test_result_setid: int, result_values: list[dict[str, Any]]
+    ) -> InlineCallbacksType[None]:
         # We're not adding support for emitting any messages, because in all cases all test results
         # will be part of a test result set. The users should wait for a 'complete' event on a
         # test result set and only then fetch the test results, which won't change from that time
