@@ -13,21 +13,90 @@
 #
 # Copyright Buildbot Team Members
 
+from __future__ import annotations
 
 import json
+from dataclasses import dataclass
+from typing import TYPE_CHECKING
 
 import sqlalchemy as sa
 from twisted.internet import defer
 
 from buildbot.db import base
 from buildbot.util import epoch2datetime
+from buildbot.util.twisted import async_to_deferred
+from buildbot.warnings import warn_deprecated
+
+if TYPE_CHECKING:
+    import datetime
+
+
+@dataclass
+class UrlModel:
+    name: str
+    url: str
+
+    # For backward compatibility
+    def __getitem__(self, key: str):
+        warn_deprecated(
+            '4.1.0',
+            (
+                'StepsConnectorComponent '
+                'getStep, and getSteps '
+                'no longer return Step as dictionnaries. '
+                'Usage of [] accessor is deprecated: please access the member directly'
+            ),
+        )
+
+        if hasattr(self, key):
+            return getattr(self, key)
+
+        raise KeyError(key)
+
+
+@dataclass
+class StepModel:
+    id: int
+    number: int
+    name: str
+    buildid: int
+    started_at: datetime.datetime | None
+    locks_acquired_at: datetime.datetime | None
+    complete_at: datetime.datetime | None
+    state_string: str
+    results: int | None
+    urls: list[UrlModel]
+    hidden: bool = False
+
+    # For backward compatibility
+    def __getitem__(self, key: str):
+        warn_deprecated(
+            '4.1.0',
+            (
+                'StepsConnectorComponent '
+                'getStep, and getSteps '
+                'no longer return Step as dictionnaries. '
+                'Usage of [] accessor is deprecated: please access the member directly'
+            ),
+        )
+
+        if hasattr(self, key):
+            return getattr(self, key)
+
+        raise KeyError(key)
 
 
 class StepsConnectorComponent(base.DBConnectorComponent):
-    url_lock = None
+    url_lock: defer.DeferredLock | None = None
 
-    @defer.inlineCallbacks
-    def getStep(self, stepid=None, buildid=None, number=None, name=None):
+    @async_to_deferred
+    async def getStep(
+        self,
+        stepid: int | None = None,
+        buildid: int | None = None,
+        number: int | None = None,
+        name: str | None = None,
+    ) -> StepModel | None:
         tbl = self.db.model.steps
         if stepid is not None:
             wc = tbl.c.id == stepid
@@ -42,34 +111,34 @@ class StepsConnectorComponent(base.DBConnectorComponent):
                 raise RuntimeError('must supply either number or name')
             wc = wc & (tbl.c.buildid == buildid)
 
-        def thd(conn):
+        def thd(conn) -> StepModel | None:
             q = self.db.model.steps.select().where(wc)
             res = conn.execute(q)
             row = res.fetchone()
 
             rv = None
             if row:
-                rv = self._stepdictFromRow(row)
+                rv = self._model_from_row(row)
             res.close()
             return rv
 
-        return (yield self.db.pool.do(thd))
+        return await self.db.pool.do(thd)
 
-    # returns a Deferred that returns a value
-    def getSteps(self, buildid):
-        def thd(conn):
+    def getSteps(self, buildid: int) -> defer.Deferred[list[StepModel]]:
+        def thd(conn) -> list[StepModel]:
             tbl = self.db.model.steps
             q = tbl.select()
             q = q.where(tbl.c.buildid == buildid)
             q = q.order_by(tbl.c.number)
             res = conn.execute(q)
-            return [self._stepdictFromRow(row) for row in res.fetchall()]
+            return [self._model_from_row(row) for row in res.fetchall()]
 
         return self.db.pool.do(thd)
 
-    # returns a Deferred that returns a value
-    def addStep(self, buildid, name, state_string):
-        def thd(conn):
+    def addStep(
+        self, buildid: int, name: str, state_string: str
+    ) -> defer.Deferred[tuple[int, int, str]]:
+        def thd(conn) -> tuple[int, int, str]:
             tbl = self.db.model.steps
             # get the highest current number
             r = conn.execute(sa.select(sa.func.max(tbl.c.number)).where(tbl.c.buildid == buildid))
@@ -91,8 +160,10 @@ class StepsConnectorComponent(base.DBConnectorComponent):
             }
             try:
                 r = conn.execute(self.db.model.steps.insert(), insert_row)
+                conn.commit()
                 got_id = r.inserted_primary_key[0]
             except (sa.exc.IntegrityError, sa.exc.ProgrammingError):
+                conn.rollback()
                 got_id = None
 
             if got_id:
@@ -112,14 +183,14 @@ class StepsConnectorComponent(base.DBConnectorComponent):
                 num += 1
             insert_row['name'] = newname
             r = conn.execute(self.db.model.steps.insert(), insert_row)
+            conn.commit()
             got_id = r.inserted_primary_key[0]
             return (got_id, number, newname)
 
         return self.db.pool.do(thd)
 
-    @defer.inlineCallbacks
-    def startStep(self, stepid, started_at, locks_acquired):
-        def thd(conn):
+    def startStep(self, stepid: int, started_at: int, locks_acquired: bool) -> defer.Deferred[None]:
+        def thd(conn) -> None:
             tbl = self.db.model.steps
             q = tbl.update().where(tbl.c.id == stepid)
             if locks_acquired:
@@ -127,27 +198,27 @@ class StepsConnectorComponent(base.DBConnectorComponent):
             else:
                 conn.execute(q.values(started_at=started_at))
 
-        yield self.db.pool.do(thd)
+        return self.db.pool.do_with_transaction(thd)
 
-    @defer.inlineCallbacks
-    def set_step_locks_acquired_at(self, stepid, locks_acquired_at):
-        def thd(conn):
+    def set_step_locks_acquired_at(
+        self, stepid: int, locks_acquired_at: int
+    ) -> defer.Deferred[None]:
+        def thd(conn) -> None:
             tbl = self.db.model.steps
             q = tbl.update().where(tbl.c.id == stepid)
             conn.execute(q.values(locks_acquired_at=locks_acquired_at))
 
-        yield self.db.pool.do(thd)
+        return self.db.pool.do_with_transaction(thd)
 
-    # returns a Deferred that returns None
-    def setStepStateString(self, stepid, state_string):
-        def thd(conn):
+    def setStepStateString(self, stepid: int, state_string: str) -> defer.Deferred[None]:
+        def thd(conn) -> None:
             tbl = self.db.model.steps
             q = tbl.update().where(tbl.c.id == stepid)
             conn.execute(q.values(state_string=state_string))
 
-        return self.db.pool.do(thd)
+        return self.db.pool.do_with_transaction(thd)
 
-    def addURL(self, stepid, name, url, _racehook=None):
+    def addURL(self, stepid: int, name: str, url: str, _racehook=None) -> defer.Deferred[None]:
         # This methods adds an URL to the db
         # This is a read modify write and thus there is a possibility
         # that several urls are added at the same time (e.g with a deferredlist
@@ -159,7 +230,7 @@ class StepsConnectorComponent(base.DBConnectorComponent):
             # this runs in reactor thread, so no race here..
             self.url_lock = defer.DeferredLock()
 
-        def thd(conn):
+        def thd(conn) -> None:
             tbl = self.db.model.steps
             wc = tbl.c.id == stepid
             q = sa.select(tbl.c.urls_json).where(wc)
@@ -173,14 +244,14 @@ class StepsConnectorComponent(base.DBConnectorComponent):
 
             if url_item not in urls:
                 urls.append(url_item)
-                q = tbl.update().where(wc)
-                conn.execute(q.values(urls_json=json.dumps(urls)))
+                q2 = tbl.update().where(wc)
+                conn.execute(q2.values(urls_json=json.dumps(urls)))
+                conn.commit()
 
-        return self.url_lock.run(lambda: self.db.pool.do(thd))
+        return self.url_lock.run(self.db.pool.do, thd)
 
-    # returns a Deferred that returns None
-    def finishStep(self, stepid, results, hidden):
-        def thd(conn):
+    def finishStep(self, stepid: int, results: int, hidden: bool) -> defer.Deferred[None]:
+        def thd(conn) -> None:
             tbl = self.db.model.steps
             q = tbl.update().where(tbl.c.id == stepid)
             conn.execute(
@@ -191,19 +262,19 @@ class StepsConnectorComponent(base.DBConnectorComponent):
                 )
             )
 
-        return self.db.pool.do(thd)
+        return self.db.pool.do_with_transaction(thd)
 
-    def _stepdictFromRow(self, row):
-        return {
-            "id": row.id,
-            "number": row.number,
-            "name": row.name,
-            "buildid": row.buildid,
-            "started_at": epoch2datetime(row.started_at),
-            "locks_acquired_at": epoch2datetime(row.locks_acquired_at),
-            "complete_at": epoch2datetime(row.complete_at),
-            "state_string": row.state_string,
-            "results": row.results,
-            "urls": json.loads(row.urls_json),
-            "hidden": bool(row.hidden),
-        }
+    def _model_from_row(self, row):
+        return StepModel(
+            id=row.id,
+            number=row.number,
+            name=row.name,
+            buildid=row.buildid,
+            started_at=epoch2datetime(row.started_at),
+            locks_acquired_at=epoch2datetime(row.locks_acquired_at),
+            complete_at=epoch2datetime(row.complete_at),
+            state_string=row.state_string,
+            results=row.results,
+            urls=[UrlModel(item['name'], item['url']) for item in json.loads(row.urls_json)],
+            hidden=bool(row.hidden),
+        )
