@@ -32,7 +32,6 @@ from buildbot.changes.manager import ChangeManager
 from buildbot.config.master import FileLoader
 from buildbot.config.master import MasterConfig
 from buildbot.data import connector as dataconnector
-from buildbot.data import graphql
 from buildbot.db import connector as dbconnector
 from buildbot.db import exceptions
 from buildbot.machine.manager import MachineManager
@@ -153,15 +152,19 @@ class BuildMaster(service.ReconfigurableServiceMixin, service.MasterService):
 
         self.workers = workermanager.WorkerManager(self)
         yield self.workers.setServiceParent(self)
+        self.workers.reconfig_priority = 127
 
         self.change_svc = ChangeManager()
         yield self.change_svc.setServiceParent(self)
 
         self.botmaster = BotMaster()
         yield self.botmaster.setServiceParent(self)
+        # must be configured first so that projects and codebases are registered
+        self.botmaster.reconfig_priority = 1001
 
         self.machine_manager = MachineManager()
         yield self.machine_manager.setServiceParent(self)
+        self.machine_manager.reconfig_priority = self.workers.reconfig_priority + 1
 
         self.scheduler_manager = SchedulerManager()
         yield self.scheduler_manager.setServiceParent(self)
@@ -181,9 +184,6 @@ class BuildMaster(service.ReconfigurableServiceMixin, service.MasterService):
         self.data = dataconnector.DataConnector()
         yield self.data.setServiceParent(self)
 
-        self.graphql = graphql.GraphQLConnector()
-        yield self.graphql.setServiceParent(self)
-
         self.www = wwwservice.WWWService()
         yield self.www.setServiceParent(self)
 
@@ -471,6 +471,7 @@ class BuildMaster(service.ReconfigurableServiceMixin, service.MasterService):
         return list(self.scheduler_manager)
 
     # state maintenance (private)
+    @defer.inlineCallbacks
     def getObjectId(self):
         """
         Return the object id for this master, for associating state with the
@@ -480,36 +481,25 @@ class BuildMaster(service.ReconfigurableServiceMixin, service.MasterService):
         """
         # try to get the cached value
         if self._object_id is not None:
-            return defer.succeed(self._object_id)
+            return self._object_id
 
         # failing that, get it from the DB; multiple calls to this function
         # at the same time will not hurt
 
-        d = self.db.state.getObjectId(self.name, "buildbot.master.BuildMaster")
-
-        @d.addCallback
-        def keep(id):
-            self._object_id = id
-            return id
-
-        return d
+        id = yield self.db.state.getObjectId(self.name, "buildbot.master.BuildMaster")
+        self._object_id = id
+        return id
 
+    @defer.inlineCallbacks
     def _getState(self, name, default=None):
         "private wrapper around C{self.db.state.getState}"
-        d = self.getObjectId()
-
-        @d.addCallback
-        def get(objectid):
-            return self.db.state.getState(objectid, name, default)
-
-        return d
+        objectid = self.getObjectId()
+        state = yield self.db.state.getState(objectid, name, default)
+        return state
 
+    @defer.inlineCallbacks
     def _setState(self, name, value):
         "private wrapper around C{self.db.state.setState}"
-        d = self.getObjectId()
-
-        @d.addCallback
-        def set(objectid):
-            return self.db.state.setState(objectid, name, value)
-
-        return d
+        objectid = yield self.getObjectId()
+        success = yield self.db.state.setState(objectid, name, value)
+        return success
