@@ -11,8 +11,9 @@
 # this program; if not, write to the Free Software Foundation, Inc., 51
 # Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
-# Copyright Buildbot Team Members
+# Copyright Buildbot Team Member
 
+from __future__ import annotations
 
 from twisted.internet import defer
 from twisted.logger import Logger
@@ -36,7 +37,7 @@ log = Logger()
 
 
 class GerritVerifyStatusPush(ReporterBase):
-    name = "GerritVerifyStatusPush"
+    name: str | None = "GerritVerifyStatusPush"  # type: ignore[assignment]
     # overridable constants
     RESULTS_TABLE = {
         SUCCESS: 1,
@@ -67,7 +68,6 @@ class GerritVerifyStatusPush(ReporterBase):
             generators = self._create_default_generators()
 
         super().checkConfig(generators=generators, **kwargs)
-        httpclientservice.HTTPClientService.checkAvailable(self.__class__.__name__)
 
     @defer.inlineCallbacks
     def reconfigService(
@@ -97,8 +97,8 @@ class GerritVerifyStatusPush(ReporterBase):
         if baseURL.endswith('/'):
             baseURL = baseURL[:-1]
 
-        self._http = yield httpclientservice.HTTPClientService.getService(
-            self.master, baseURL, auth=auth, debug=self.debug, verify=self.verify
+        self._http = yield httpclientservice.HTTPSession(
+            self.master.httpservice, baseURL, auth=auth, debug=self.debug, verify=self.verify
         )
 
         self._verification_name = verification_name or Interpolate('%(prop:buildername)s')
