@@ -17,7 +17,6 @@ import re
 
 
 class RevlinkMatch:
-
     def __init__(self, repo_urls, revlink):
         if isinstance(repo_urls, str):
             repo_urls = [repo_urls]
@@ -33,27 +32,38 @@ class RevlinkMatch:
 
 
 GithubRevlink = RevlinkMatch(
-    repo_urls=[r'https://github.com/([^/]*)/([^/]*?)(?:\.git)?$',
-               r'git://github.com/([^/]*)/([^/]*?)(?:\.git)?$',
-               r'git@github.com:([^/]*)/([^/]*?)(?:\.git)?$',
-               r'ssh://git@github.com/([^/]*)/([^/]*?)(?:\.git)?$'
-               ],
-    revlink=r'https://github.com/\1/\2/commit/%s')
+    repo_urls=[
+        r'https://github.com/([^/]*)/([^/]*?)(?:\.git)?$',
+        r'git://github.com/([^/]*)/([^/]*?)(?:\.git)?$',
+        r'git@github.com:([^/]*)/([^/]*?)(?:\.git)?$',
+        r'ssh://git@github.com/([^/]*)/([^/]*?)(?:\.git)?$',
+    ],
+    revlink=r'https://github.com/\1/\2/commit/%s',
+)
+
+
+BitbucketRevlink = RevlinkMatch(
+    repo_urls=[
+        r'https://[^@]*@bitbucket.org/([^/]*)/([^/]*?)(?:\.git)?$',
+        r'git@bitbucket.org:([^/]*)/([^/]*?)(?:\.git)?$',
+    ],
+    revlink=r'https://bitbucket.org/\1/\2/commits/%s',
+)
 
 
 class GitwebMatch(RevlinkMatch):
-
     def __init__(self, repo_urls, revlink):
-        super().__init__(repo_urls=repo_urls,
-                         revlink=revlink + r'?p=\g<repo>;a=commit;h=%s')
+        super().__init__(repo_urls=repo_urls, revlink=revlink + r'?p=\g<repo>;a=commit;h=%s')
 
 
 SourceforgeGitRevlink = GitwebMatch(
-    repo_urls=[r'^git://([^.]*).git.sourceforge.net/gitroot/(?P<repo>.*)$',
-               r'[^@]*@([^.]*).git.sourceforge.net:gitroot/(?P<repo>.*)$',
-               r'ssh://(?:[^@]*@)?([^.]*).git.sourceforge.net/gitroot/(?P<repo>.*)$',
-               ],
-    revlink=r'http://\1.git.sourceforge.net/git/gitweb.cgi')
+    repo_urls=[
+        r'^git://([^.]*).git.sourceforge.net/gitroot/(?P<repo>.*)$',
+        r'[^@]*@([^.]*).git.sourceforge.net:gitroot/(?P<repo>.*)$',
+        r'ssh://(?:[^@]*@)?([^.]*).git.sourceforge.net/gitroot/(?P<repo>.*)$',
+    ],
+    revlink=r'http://\1.git.sourceforge.net/git/gitweb.cgi',
+)
 
 # SourceForge recently upgraded to another platform called Allura
 # See introduction:
@@ -61,15 +71,16 @@ SourceforgeGitRevlink = GitwebMatch(
 # And as reference:
 # https://sourceforge.net/p/forge/community-docs/SVN%20and%20project%20upgrades/
 SourceforgeGitRevlink_AlluraPlatform = RevlinkMatch(
-    repo_urls=[r'git://git.code.sf.net/p/(?P<repo>.*)$',
-               r'http://git.code.sf.net/p/(?P<repo>.*)$',
-               r'ssh://(?:[^@]*@)?git.code.sf.net/p/(?P<repo>.*)$'
-               ],
-    revlink=r'https://sourceforge.net/p/\1/ci/%s/')
+    repo_urls=[
+        r'git://git.code.sf.net/p/(?P<repo>.*)$',
+        r'http://git.code.sf.net/p/(?P<repo>.*)$',
+        r'ssh://(?:[^@]*@)?git.code.sf.net/p/(?P<repo>.*)$',
+    ],
+    revlink=r'https://sourceforge.net/p/\1/ci/%s/',
+)
 
 
 class RevlinkMultiplexer:
-
     def __init__(self, *revlinks):
         self.revlinks = revlinks
 
@@ -81,6 +92,6 @@ class RevlinkMultiplexer:
         return None
 
 
-default_revlink_matcher = RevlinkMultiplexer(GithubRevlink,
-                                             SourceforgeGitRevlink,
-                                             SourceforgeGitRevlink_AlluraPlatform)
+default_revlink_matcher = RevlinkMultiplexer(
+    GithubRevlink, BitbucketRevlink, SourceforgeGitRevlink, SourceforgeGitRevlink_AlluraPlatform
+)
