@@ -12,6 +12,7 @@
 # Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 # Copyright Buildbot Team Members
+from __future__ import annotations
 
 import re
 
@@ -138,13 +139,13 @@ class SetPropertyFromCommand(buildstep.ShellMixin, buildstep.BuildStep):
                 self.setProperty(k, v, "SetPropertyFromCommand Step")
             property_changes = new_props
 
-        props_set = [f"{k}: {repr(v)}" for k, v in sorted(property_changes.items())]
+        props_set = [f"{k}: {v!r}" for k, v in sorted(property_changes.items())]
         yield self.addCompleteLog('property changes', "\n".join(props_set))
 
         if len(property_changes) > 1:
             self.descriptionDone = f'{len(property_changes)} properties set'
         elif len(property_changes) == 1:
-            self.descriptionDone = f'property \'{list(property_changes)[0]}\' set'
+            self.descriptionDone = f'property \'{next(iter(property_changes))}\' set'
         if cmd.didFail():
             return FAILURE
         return SUCCESS
@@ -160,25 +161,27 @@ class ShellCommand(buildstep.ShellMixin, buildstep.BuildStep):
 
             # check validity of arguments being passed to RemoteShellCommand
             valid_rsc_args = [
-                'command',
-                'env',
-                'want_stdout',
-                'want_stderr',
-                'timeout',
-                'maxTime',
-                'sigtermTime',
-                'logfiles',
-                'lazylogfiles',
-                'usePTY',
-                'logEnviron',
-                'collectStdout',
-                'collectStderr',
-                'interruptSignal',
-                'initialStdin',
-                'decodeRC',
-                'stdioLogName',
-                'workdir',
-            ] + buildstep.BuildStep._params_names
+                "command",
+                "env",
+                "want_stdout",
+                "want_stderr",
+                "timeout",
+                "maxTime",
+                "max_lines",
+                "sigtermTime",
+                "logfiles",
+                "lazylogfiles",
+                "usePTY",
+                "logEnviron",
+                "collectStdout",
+                "collectStderr",
+                "interruptSignal",
+                "initialStdin",
+                "decodeRC",
+                "stdioLogName",
+                "workdir",
+                *buildstep.BuildStep._params_names,
+            ]
 
             invalid_args = []
             for arg in kwargs:
@@ -202,8 +205,8 @@ class ShellCommand(buildstep.ShellMixin, buildstep.BuildStep):
 
 class Configure(ShellCommand):
     name = "configure"
-    haltOnFailure = 1
-    flunkOnFailure = 1
+    haltOnFailure = True
+    flunkOnFailure = True
     description = "configuring"
     descriptionDone = "configure"
     command = ["./configure"]
@@ -224,7 +227,7 @@ class WarningCountingShellCommand(buildstep.ShellMixin, CompositeStepMixin, buil
     # The defaults work for GNU Make.
     directoryEnterPattern = "make.*: Entering directory [\u2019\"`'](.*)[\u2019'`\"]"
     directoryLeavePattern = "make.*: Leaving directory"
-    suppressionFile = None
+    suppressionFile: str | None = None
 
     commentEmptyLineRe = re.compile(r"^\s*(#.*)?$")
     suppressionLineRe = re.compile(r"^\s*(.+?)\s*:\s*(.+?)\s*(?:[:]\s*([0-9]+)(?:-([0-9]+))?\s*)?$")
@@ -464,8 +467,8 @@ class WarningCountingShellCommand(buildstep.ShellMixin, CompositeStepMixin, buil
 
 class Compile(WarningCountingShellCommand):
     name = "compile"
-    haltOnFailure = 1
-    flunkOnFailure = 1
+    haltOnFailure = True
+    flunkOnFailure = True
     description = ["compiling"]
     descriptionDone = ["compile"]
     command = ["make", "all"]
@@ -473,7 +476,7 @@ class Compile(WarningCountingShellCommand):
 
 class Test(WarningCountingShellCommand):
     name = "test"
-    warnOnFailure = 1
+    warnOnFailure = True
     description = ["testing"]
     descriptionDone = ["test"]
     command = ["make", "test"]
