@@ -52,7 +52,7 @@ class SetPropertiesFromEnv(WorkerBuildStep):
         # a case-sensitive dictionary in worker_environ.  Fortunately, that
         # dictionary is also folded to uppercase, so we can simply fold the
         # variable names to uppercase to duplicate the case-insensitivity.
-        fold_to_uppercase = self.worker.worker_system == 'win32'
+        fold_to_uppercase = self.worker.worker_system == 'nt'
 
         properties = self.build.getProperties()
         environ = self.worker.worker_environ
@@ -68,7 +68,7 @@ class SetPropertiesFromEnv(WorkerBuildStep):
             if value:
                 # note that the property is not uppercased
                 properties.setProperty(variable, value, self.source, runtime=True)
-                log.append(f"{variable} = {repr(value)}")
+                log.append(f"{variable} = {value!r}")
         yield self.addCompleteLog("properties", "\n".join(log))
         return SUCCESS
 
@@ -225,6 +225,7 @@ class CompositeStepMixin:
         self.rc_log = yield self.addLog(logname)
         return self.rc_log
 
+    @defer.inlineCallbacks
     def runRemoteCommand(
         self, cmd, args, abandonOnFailure=True, evaluateCommand=lambda cmd: cmd.didFail()
     ):
@@ -232,15 +233,11 @@ class CompositeStepMixin:
         cmd = remotecommand.RemoteCommand(cmd, args)
         if hasattr(self, "rc_log"):
             cmd.useLog(self.rc_log, False)
-        d = self.runCommand(cmd)
-
-        def commandComplete(cmd):
-            if abandonOnFailure and cmd.didFail():
-                raise buildstep.BuildStepFailed()
-            return evaluateCommand(cmd)
-
-        d.addCallback(lambda res: commandComplete(cmd))
-        return d
+        yield self.runCommand(cmd)
+        if abandonOnFailure and cmd.didFail():
+            raise buildstep.BuildStepFailed()
+        res = yield evaluateCommand(cmd)
+        return res
 
     def runRmdir(self, dir, timeout=None, **kwargs):
         """remove a directory from the worker"""
