@@ -15,10 +15,11 @@
 
 
 import json as jsonmodule
-from unittest import mock
 
 from twisted.internet import defer
 from twisted.logger import Logger
+from twisted.python import deprecate
+from twisted.python import versions
 from zope.interface import implementer
 
 from buildbot import util
@@ -27,6 +28,7 @@ from buildbot.util import httpclientservice
 from buildbot.util import service
 from buildbot.util import toJson
 from buildbot.util import unicode2bytes
+from buildbot.util.twisted import async_to_deferred
 
 log = Logger()
 
@@ -68,25 +70,28 @@ class HTTPClientService(service.SharedService):
     quiet = False
 
     def __init__(
-        self, base_url, auth=None, headers=None, debug=None, verify=None, skipEncoding=None
+        self, base_url, auth=None, headers=None, debug=False, verify=None, skipEncoding=False
     ):
         assert not base_url.endswith("/"), "baseurl should not end with /"
         super().__init__()
-        self._base_url = base_url
-        self._auth = auth
+        self._session = httpclientservice.HTTPSession(
+            self,
+            base_url,
+            auth=auth,
+            headers=headers,
+            debug=debug,
+            verify=verify,
+            skip_encoding=skipEncoding,
+        )
 
-        self._headers = headers
-        self._session = None
         self._expected = []
 
     def updateHeaders(self, headers):
-        if self._headers is None:
-            self._headers = {}
-        self._headers.update(headers)
+        self._session.update_headers(headers)
 
     @classmethod
-    @defer.inlineCallbacks
-    def getService(cls, master, case, *args, **kwargs):
+    @async_to_deferred
+    async def getService(cls, master, case, *args, **kwargs):
         def assertNotCalled(self, *_args, **_kwargs):
             case.fail(
                 f"HTTPClientService called with *{_args!r}, **{_kwargs!r} "
@@ -95,18 +100,19 @@ class HTTPClientService(service.SharedService):
 
         case.patch(httpclientservice.HTTPClientService, "__init__", assertNotCalled)
 
-        service = yield super().getService(master, *args, **kwargs)
+        service = await super().getService(master, *args, **kwargs)
         service.case = case
         case.addCleanup(service.assertNoOutstanding)
-        return service
 
-    # tests should ensure this has been called
-    checkAvailable = mock.Mock()
+        master.httpservice = service
+
+        return service
 
     def expect(
         self,
         method,
         ep,
+        session=None,
         params=None,
         headers=None,
         data=None,
@@ -127,6 +133,7 @@ class HTTPClientService(service.SharedService):
 
         self._expected.append({
             "method": method,
+            "session": session,
             "ep": ep,
             "params": params,
             "headers": headers,
@@ -146,20 +153,25 @@ class HTTPClientService(service.SharedService):
             0, len(self._expected), f"expected more http requests:\n {self._expected!r}"
         )
 
-    @defer.inlineCallbacks
-    def _doRequest(
+    @async_to_deferred
+    async def _do_request(
         self,
+        session,
         method,
         ep,
         params=None,
         headers=None,
+        cookies=None,  # checks are not implemented
         data=None,
         json=None,
         files=None,
+        auth=None,  # checks are not implemented
         timeout=None,
         verify=None,
         cert=None,
-    ):
+        allow_redirects=None,  # checks are not implemented
+        proxies=None,  # checks are not implemented
+    ) -> IHttpResponse:
         if ep.startswith('http://') or ep.startswith('https://'):
             pass
         else:
@@ -187,9 +199,17 @@ class HTTPClientService(service.SharedService):
         expect = self._expected.pop(0)
         processing_delay_s = expect.pop("processing_delay_s")
 
+        expect_session = expect["session"] or self._session
+
         # pylint: disable=too-many-boolean-expressions
         if (
-            expect["method"] != method
+            expect_session.base_url != session.base_url
+            or expect_session.auth != session.auth
+            or expect_session.headers != session.headers
+            or expect_session.verify != session.verify
+            or expect_session.debug != session.debug
+            or expect_session.skip_encoding != session.skip_encoding
+            or expect["method"] != method
             or expect["ep"] != ep
             or expect["params"] != params
             or expect["headers"] != headers
@@ -201,6 +221,12 @@ class HTTPClientService(service.SharedService):
         ):
             raise AssertionError(
                 "expecting:\n"
+                f"session.base_url={expect_session.base_url!r}, "
+                f"session.auth={expect_session.auth!r}, "
+                f"session.headers={expect_session.headers!r}, "
+                f"session.verify={expect_session.verify!r}, "
+                f"session.debug={expect_session.debug!r}, "
+                f"session.skip_encoding={expect_session.skip_encoding!r}, "
                 f"method={expect['method']!r}, "
                 f"ep={expect['ep']!r}, "
                 f"params={expect['params']!r}, "
@@ -211,6 +237,12 @@ class HTTPClientService(service.SharedService):
                 f"verify={expect['verify']!r}, "
                 f"cert={expect['cert']!r}"
                 "\ngot      :\n"
+                f"session.base_url={session.base_url!r}, "
+                f"session.auth={session.auth!r}, "
+                f"session.headers={session.headers!r}, "
+                f"session.verify={session.verify!r}, "
+                f"session.debug={session.debug!r}, "
+                f"session.skip_encoding={session.skip_encoding!r}, "
                 f"method={method!r}, "
                 f"ep={ep!r}, "
                 f"params={params!r}, "
@@ -231,19 +263,23 @@ class HTTPClientService(service.SharedService):
             )
 
         if processing_delay_s is not None:
-            yield util.asyncSleep(1, reactor=self.master.reactor)
+            await util.asyncSleep(1, reactor=self.master.reactor)
 
         return ResponseWrapper(expect['code'], expect['content'])
 
     # lets be nice to the auto completers, and don't generate that code
+    @deprecate.deprecated(versions.Version("buildbot", 4, 1, 0))
     def get(self, ep, **kwargs):
-        return self._doRequest('get', ep, **kwargs)
+        return self._do_request(self._session, 'get', ep, **kwargs)
 
+    @deprecate.deprecated(versions.Version("buildbot", 4, 1, 0))
     def put(self, ep, **kwargs):
-        return self._doRequest('put', ep, **kwargs)
+        return self._do_request(self._session, 'put', ep, **kwargs)
 
+    @deprecate.deprecated(versions.Version("buildbot", 4, 1, 0))
     def delete(self, ep, **kwargs):
-        return self._doRequest('delete', ep, **kwargs)
+        return self._do_request(self._session, 'delete', ep, **kwargs)
 
+    @deprecate.deprecated(versions.Version("buildbot", 4, 1, 0))
     def post(self, ep, **kwargs):
-        return self._doRequest('post', ep, **kwargs)
+        return self._do_request(self._session, 'post', ep, **kwargs)
