@@ -13,13 +13,23 @@
 #
 # Copyright Buildbot Team Members
 
+from typing import TYPE_CHECKING
 
 from twisted.internet import defer
+from twisted.trial import unittest
 
+from buildbot.test import fakedb
 from buildbot.test.fake import fakemaster
 
+if TYPE_CHECKING:
+    from twisted.trial import unittest
 
-class ChangeSourceMixin:
+    _ChangeSourceMixinBase = unittest.TestCase
+else:
+    _ChangeSourceMixinBase = object
+
+
+class ChangeSourceMixin(_ChangeSourceMixinBase):
     """
     This class is used for testing change sources, and handles a few things:
 
@@ -40,7 +50,12 @@ class ChangeSourceMixin:
         self.master = yield fakemaster.make_master(
             self, wantDb=True, wantData=True, wantRealReactor=want_real_reactor
         )
-        assert not hasattr(self.master, 'addChange')  # just checking..
+
+        self.master.db.insert_test_data([
+            fakedb.Master(id=fakedb.FakeDBConnector.MASTER_ID),
+            fakedb.Master(id=self.OTHER_MASTER_ID, active=1),
+            fakedb.ChangeSource(id=self.DUMMY_CHANGESOURCE_ID, name=self.DEFAULT_NAME),
+        ])
 
         @defer.inlineCallbacks
         def cleanup():
@@ -71,17 +86,9 @@ class ChangeSourceMixin:
 
         self.started = False
 
+    @defer.inlineCallbacks
     def setChangeSourceToMaster(self, otherMaster):
-        # some tests build the CS late, so for those tests we will require that
-        # they use the default name in order to run tests that require master
-        # assignments
-        if self.changesource is not None:
-            name = self.changesource.name
-        else:
-            name = self.DEFAULT_NAME
-
-        self.master.data.updates.changesourceIds[name] = self.DUMMY_CHANGESOURCE_ID
-        if otherMaster:
-            self.master.data.updates.changesourceMasters[self.DUMMY_CHANGESOURCE_ID] = otherMaster
-        else:
-            del self.master.data.updates.changesourceMasters[self.DUMMY_CHANGESOURCE_ID]
+        ret = yield self.master.data.updates.trySetChangeSourceMaster(
+            self.DUMMY_CHANGESOURCE_ID, otherMaster
+        )
+        return ret
