@@ -49,7 +49,7 @@ def _get_target(target):
     try:
         target, attribute = target.rsplit('.', 1)
     except (TypeError, ValueError) as e:
-        raise TypeError(f"Need a valid target to patch. You supplied: {repr(target)}") from e
+        raise TypeError(f"Need a valid target to patch. You supplied: {target!r}") from e
     return _importer(target), attribute
 
 
@@ -76,6 +76,7 @@ class DelayWrapper:
 def patchForDelay(target_name):
     class Default:
         pass
+
     default = Default()
 
     target, attribute = _get_target(target_name)
