@@ -68,12 +68,9 @@ class SchedulerMixin(interfaces.InterfaceTests):
         """
         scheduler.objectid = objectid
 
-        # set up a fake master
-        db = self.db = self.master.db
-        self.mq = self.master.mq
-        scheduler.setServiceParent(self.master)
-
         rows = [
+            fakedb.Master(id=fakedb.FakeDBConnector.MASTER_ID),
+            fakedb.Master(id=self.OTHER_MASTER_ID),
             fakedb.Scheduler(id=schedulerid, name=scheduler.name),
         ]
         if createBuilderDB is True:
@@ -82,7 +79,9 @@ class SchedulerMixin(interfaces.InterfaceTests):
                 for i, bname in enumerate(scheduler.builderNames)
             ])
 
-        yield db.insert_test_data(rows)
+        yield self.master.db.insert_test_data(rows)
+
+        yield scheduler.setServiceParent(self.master)
 
         if overrideBuildsetMethods:
             self.assertArgSpecMatches(
@@ -118,17 +117,17 @@ class SchedulerMixin(interfaces.InterfaceTests):
                 self.addedSourceStamps.append(kwargs)
                 return defer.succeed(300 + len(self.addedSourceStamps) - 1)
 
-            self.db.sourcestamps.addSourceStamp = fake_addSourceStamp
+            self.master.db.sourcestamps.addSourceStamp = fake_addSourceStamp
 
             def fake_addSourceStampSet():
                 self.addedSourceStampSets.append([])
                 return defer.succeed(400 + len(self.addedSourceStampSets) - 1)
 
-            self.db.sourcestamps.addSourceStampSet = fake_addSourceStampSet
+            self.master.db.sourcestamps.addSourceStampSet = fake_addSourceStampSet
 
         # patch methods to detect a failure to upcall the activate and
         # deactivate methods .. unless we're testing BaseScheduler
-        def patch(meth):
+        def patch(scheduler_class, meth):
             oldMethod = getattr(scheduler, meth)
 
             @defer.inlineCallbacks
@@ -141,18 +140,35 @@ class SchedulerMixin(interfaces.InterfaceTests):
 
             setattr(scheduler, meth, newMethod)
 
-            oldParent = getattr(base.BaseScheduler, meth)
+            oldParent = getattr(scheduler_class, meth)
 
             def newParent(self_):
                 self._parentMethodCalled = True
                 return oldParent(self_)
 
             self.patch(base.BaseScheduler, meth, newParent)
+            self.patch(base.ReconfigurableBaseScheduler, meth, newParent)
 
-        if scheduler.__class__.activate != base.BaseScheduler.activate:
-            patch('activate')
-        if scheduler.__class__.deactivate != base.BaseScheduler.deactivate:
-            patch('deactivate')
+        if (
+            isinstance(scheduler, base.BaseScheduler)
+            and scheduler.__class__.activate != base.BaseScheduler.activate
+        ):
+            patch(base.BaseScheduler, 'activate')
+        if (
+            isinstance(scheduler, base.BaseScheduler)
+            and scheduler.__class__.deactivate != base.BaseScheduler.deactivate
+        ):
+            patch(base.BaseScheduler, 'deactivate')
+        if (
+            isinstance(scheduler, base.ReconfigurableBaseScheduler)
+            and scheduler.__class__.activate != base.ReconfigurableBaseScheduler.activate
+        ):
+            patch(base.ReconfigurableBaseScheduler, 'activate')
+        if (
+            isinstance(scheduler, base.ReconfigurableBaseScheduler)
+            and scheduler.__class__.deactivate != base.ReconfigurableBaseScheduler.deactivate
+        ):
+            patch(base.ReconfigurableBaseScheduler, 'deactivate')
 
         self.sched = scheduler
         return scheduler
@@ -160,10 +176,7 @@ class SchedulerMixin(interfaces.InterfaceTests):
     @defer.inlineCallbacks
     def setSchedulerToMaster(self, otherMaster):
         sched_id = yield self.master.data.updates.findSchedulerId(self.sched.name)
-        if otherMaster:
-            self.master.data.updates.schedulerMasters[sched_id] = otherMaster
-        else:
-            del self.master.data.updates.schedulerMasters[sched_id]
+        yield self.master.data.updates.trySetSchedulerMaster(sched_id, otherMaster)
 
     class FakeChange:
         who = ''
@@ -219,7 +232,7 @@ class SchedulerMixin(interfaces.InterfaceTests):
         if builderNames is None:
             builderNames = self.sched.builderNames
         builderids = []
-        builders = yield self.db.builders.getBuilders()
+        builders = yield self.master.db.builders.getBuilders()
         for builderName in builderNames:
             for bldrDict in builders:
                 if builderName == bldrDict.name:
