@@ -14,6 +14,7 @@
 # Copyright Buildbot Team Members
 
 # See "Type Validation" in master/docs/developer/tests.rst
+from __future__ import annotations
 
 import datetime
 import json
@@ -28,7 +29,7 @@ validatorsByName = {}
 
 
 class Validator:
-    name = None
+    name: str | None = None
     hasArgs = False
 
     def validate(self, name, object):
@@ -37,7 +38,7 @@ class Validator:
     class __metaclass__(type):
         def __new__(mcs, name, bases, attrs):
             cls = type.__new__(mcs, name, bases, attrs)
-            if 'name' in attrs and attrs['name']:
+            if attrs.get('name'):
                 assert attrs['name'] not in validatorsByName
                 validatorsByName[attrs['name']] = cls
             return cls
@@ -47,7 +48,7 @@ class Validator:
 
 
 class InstanceValidator(Validator):
-    types = ()
+    types: tuple[type] | tuple[()] = ()
 
     def validate(self, name, object):
         if not isinstance(object, self.types):
@@ -166,13 +167,13 @@ class DictValidator(Validator):
 
 
 class SequenceValidator(Validator):
-    type = None
+    type: type | None = None
 
     def __init__(self, elementValidator):
         self.elementValidator = elementValidator
 
     def validate(self, name, object):
-        if not isinstance(object, self.type):  # noqa pylint: disable=isinstance-second-argument-not-valid-type
+        if not isinstance(object, self.type):
             yield f"{name} ({object!r}) is not a {self.name}"
             return
 
@@ -230,7 +231,7 @@ class JsonValidator(Validator):
 
 
 class PatchValidator(Validator):
-    name = 'patch'
+    name: str | None = 'patch'  # type: ignore[assignment]
 
     validator = DictValidator(
         body=NoneOk(BinaryValidator()),
@@ -291,7 +292,6 @@ class Selector(Validator):
 # Type definitions
 
 message = {}
-dbdict = {}
 
 # parse and use a ResourceType class's dataFields into a validator
 
@@ -311,13 +311,6 @@ message['masters'].add(
     ),
 )
 
-dbdict['masterdict'] = DictValidator(
-    id=IntValidator(),
-    name=StringValidator(),
-    active=BooleanValidator(),
-    last_active=DateTimeValidator(),
-)
-
 # sourcestamp
 
 _sourcestamp = {
@@ -342,32 +335,6 @@ _sourcestamp = {
 message['sourcestamps'] = Selector()
 message['sourcestamps'].add(None, DictValidator(**_sourcestamp))
 
-dbdict['ssdict'] = DictValidator(
-    ssid=IntValidator(),
-    branch=NoneOk(StringValidator()),
-    revision=NoneOk(StringValidator()),
-    patchid=NoneOk(IntValidator()),
-    patch_body=NoneOk(BinaryValidator()),
-    patch_level=NoneOk(IntValidator()),
-    patch_subdir=NoneOk(StringValidator()),
-    patch_author=NoneOk(StringValidator()),
-    patch_comment=NoneOk(StringValidator()),
-    codebase=StringValidator(),
-    repository=StringValidator(),
-    project=StringValidator(),
-    created_at=DateTimeValidator(),
-)
-
-# project
-dbdict['projectdict'] = DictValidator(
-    id=IntValidator(),
-    name=StringValidator(),
-    slug=StringValidator(),
-    description=NoneOk(StringValidator()),
-    description_format=NoneOk(StringValidator()),
-    description_html=NoneOk(StringValidator()),
-)
-
 # builder
 
 message['builders'] = Selector()
@@ -383,35 +350,6 @@ message['builders'].add(
     ),
 )
 
-dbdict['builderdict'] = DictValidator(
-    id=IntValidator(),
-    masterids=ListValidator(IntValidator()),
-    name=StringValidator(),
-    description=NoneOk(StringValidator()),
-    description_format=NoneOk(StringValidator()),
-    description_html=NoneOk(StringValidator()),
-    projectid=NoneOk(IntValidator()),
-    tags=ListValidator(StringValidator()),
-)
-
-# worker
-
-dbdict['workerdict'] = DictValidator(
-    id=IntValidator(),
-    name=StringValidator(),
-    configured_on=ListValidator(
-        DictValidator(
-            masterid=IntValidator(),
-            builderid=IntValidator(),
-        )
-    ),
-    paused=BooleanValidator(),
-    pause_reason=NoneOk(StringValidator()),
-    graceful=BooleanValidator(),
-    connected_to=ListValidator(IntValidator()),
-    workerinfo=JsonValidator(),
-)
-
 # buildset
 
 _buildset = {
@@ -449,20 +387,6 @@ message['buildsets'].add(
     ),
 )
 
-dbdict['bsdict'] = DictValidator(
-    bsid=IntValidator(),
-    external_idstring=NoneOk(StringValidator()),
-    reason=StringValidator(),
-    sourcestamps=ListValidator(IntValidator()),
-    submitted_at=DateTimeValidator(),
-    complete=BooleanValidator(),
-    complete_at=NoneOk(DateTimeValidator()),
-    results=NoneOk(IntValidator()),
-    rebuilt_buildid=NoneOk(IntValidator()),
-    parent_buildid=NoneOk(IntValidator()),
-    parent_relationship=NoneOk(StringValidator()),
-)
-
 # buildrequest
 
 message['buildrequests'] = Selector()
@@ -508,42 +432,6 @@ message['changes'].add(
     ),
 )
 
-dbdict['chdict'] = DictValidator(
-    changeid=IntValidator(),
-    author=StringValidator(),
-    committer=StringValidator(),
-    files=ListValidator(StringValidator()),
-    comments=StringValidator(),
-    revision=NoneOk(StringValidator()),
-    when_timestamp=DateTimeValidator(),
-    branch=NoneOk(StringValidator()),
-    category=NoneOk(StringValidator()),
-    revlink=NoneOk(StringValidator()),
-    properties=SourcedPropertiesValidator(),
-    repository=StringValidator(),
-    project=StringValidator(),
-    codebase=StringValidator(),
-    sourcestampid=IntValidator(),
-    parent_changeids=ListValidator(IntValidator()),
-)
-
-# changesources
-
-dbdict['changesourcedict'] = DictValidator(
-    id=IntValidator(),
-    name=StringValidator(),
-    masterid=NoneOk(IntValidator()),
-)
-
-# schedulers
-
-dbdict['schedulerdict'] = DictValidator(
-    id=IntValidator(),
-    name=StringValidator(),
-    masterid=NoneOk(IntValidator()),
-    enabled=BooleanValidator(),
-)
-
 # builds
 
 _build = {
@@ -566,27 +454,7 @@ message['builds'].add(
     None, MessageValidator(events=_buildEvents, messageValidator=DictValidator(**_build))
 )
 
-# As build's properties are fetched at DATA API level,
-# a distinction shall be made as both are not equal.
-# Validates DB layer
-dbdict['dbbuilddict'] = buildbase = DictValidator(
-    id=IntValidator(),
-    number=IntValidator(),
-    builderid=IntValidator(),
-    buildrequestid=IntValidator(),
-    workerid=IntValidator(),
-    masterid=IntValidator(),
-    started_at=DateTimeValidator(),
-    complete_at=NoneOk(DateTimeValidator()),
-    locks_duration_s=IntValidator(),
-    state_string=StringValidator(),
-    results=NoneOk(IntValidator()),
-)
-
 # Validates DATA API layer
-dbdict['builddict'] = DictValidator(
-    properties=NoneOk(SourcedPropertiesValidator()), **buildbase.keys
-)
 
 # build data
 
@@ -601,14 +469,6 @@ _build_data_msgdict = DictValidator(
 message['build_data'] = Selector()
 message['build_data'].add(None, MessageValidator(events=[], messageValidator=_build_data_msgdict))
 
-dbdict['build_datadict'] = DictValidator(
-    buildid=IntValidator(),
-    name=StringValidator(),
-    value=NoneOk(BinaryValidator()),
-    length=IntValidator(),
-    source=StringValidator(),
-)
-
 # steps
 
 _step = {
@@ -631,20 +491,6 @@ message['steps'].add(
     None, MessageValidator(events=_stepEvents, messageValidator=DictValidator(**_step))
 )
 
-dbdict['stepdict'] = DictValidator(
-    id=IntValidator(),
-    number=IntValidator(),
-    name=IdentifierValidator(50),
-    buildid=IntValidator(),
-    started_at=DateTimeValidator(),
-    locks_acquired_at=NoneOk(DateTimeValidator()),
-    complete_at=NoneOk(DateTimeValidator()),
-    state_string=StringValidator(),
-    results=NoneOk(IntValidator()),
-    urls=ListValidator(StringValidator()),
-    hidden=BooleanValidator(),
-)
-
 # logs
 
 _log = {
@@ -657,18 +503,6 @@ _log = {
 }
 _logEvents = ['new', 'complete', 'appended']
 
-# message['log']
-
-dbdict['logdict'] = DictValidator(
-    id=IntValidator(),
-    stepid=IntValidator(),
-    name=StringValidator(),
-    slug=IdentifierValidator(50),
-    complete=BooleanValidator(),
-    num_lines=IntValidator(),
-    type=IdentifierValidator(1),
-)
-
 # test results sets
 
 _test_result_set_msgdict = DictValidator(
@@ -688,19 +522,6 @@ message['test_result_sets'].add(
     None, MessageValidator(events=[b'new', b'completed'], messageValidator=_test_result_set_msgdict)
 )
 
-dbdict['test_result_setdict'] = DictValidator(
-    id=IntValidator(),
-    builderid=IntValidator(),
-    buildid=IntValidator(),
-    stepid=IntValidator(),
-    description=NoneOk(StringValidator()),
-    category=StringValidator(),
-    value_unit=StringValidator(),
-    tests_passed=NoneOk(IntValidator()),
-    tests_failed=NoneOk(IntValidator()),
-    complete=BooleanValidator(),
-)
-
 # test results
 
 _test_results_msgdict = DictValidator(
@@ -718,17 +539,6 @@ message['test_results'].add(
     None, MessageValidator(events=[b'new'], messageValidator=_test_results_msgdict)
 )
 
-dbdict['test_resultdict'] = DictValidator(
-    id=IntValidator(),
-    builderid=IntValidator(),
-    test_result_setid=IntValidator(),
-    test_name=NoneOk(StringValidator()),
-    test_code_path=NoneOk(StringValidator()),
-    line=NoneOk(IntValidator()),
-    duration_ns=NoneOk(IntValidator()),
-    value=StringValidator(),
-)
-
 
 # external functions
 
@@ -753,10 +563,6 @@ def verifyMessage(testcase, routingKey, message_):
     _verify(testcase, validator, '', (routingKey, (routingKey, message_)))
 
 
-def verifyDbDict(testcase, type, value):
-    _verify(testcase, dbdict[type], type, value)
-
-
 def verifyData(testcase, entityType, options, value):
     _verify(testcase, entityType, entityType.name, value)
 
