@@ -13,11 +13,15 @@
 #
 # Copyright Buildbot Team Members
 
+from __future__ import annotations
+
 import json
 import os
 from importlib.metadata import entry_points
 from io import BytesIO
 from io import StringIO
+from typing import TYPE_CHECKING
+from typing import Any
 from unittest import mock
 from urllib.parse import parse_qs
 from urllib.parse import unquote as urlunquote
@@ -32,6 +36,10 @@ from buildbot.util import unicode2bytes
 from buildbot.util.importlib_compat import entry_points_get
 from buildbot.www import auth
 from buildbot.www import authz
+from buildbot.www.authz.authz import Authz
+
+if TYPE_CHECKING:
+    from buildbot.util.twisted import InlineCallbacksType
 
 
 class FakeSession:
@@ -53,6 +61,9 @@ class FakeRequest:
     responseCode = 200
 
     def __init__(self, path=None):
+        # from twisted.web.http.Request. Used to detect connection dropped
+        self.channel = True
+
         self.headers = {}
         self.input_headers = {}
         self.prepath = []
@@ -135,8 +146,11 @@ class RequiresWwwMixin:
 class WwwTestMixin(RequiresWwwMixin):
     UUID = str(uuid1())
 
-    def make_master(self, wantGraphql=False, url=None, **kwargs):
-        master = fakemaster.make_master(self, wantData=True, wantGraphql=wantGraphql)
+    @defer.inlineCallbacks
+    def make_master(
+        self, wantGraphql: bool = False, url: str | None = None, **kwargs: Any
+    ) -> InlineCallbacksType[fakemaster.FakeMaster]:
+        master = yield fakemaster.make_master(self, wantData=True, wantGraphql=wantGraphql)
         self.master = master
         master.www = mock.Mock()  # to handle the resourceNeedsReconfigs call
         master.www.getUserInfos = lambda _: getattr(
@@ -149,6 +163,7 @@ class WwwTestMixin(RequiresWwwMixin):
             master.config.buildbotURL = url
         self.master.session = FakeSession()
         self.master.authz = cfg["authz"]
+        assert isinstance(self.master.authz, Authz)
         self.master.authz.setMaster(self.master)
         return master
 
