@@ -17,8 +17,9 @@ import re
 
 from buildbot import util
 
-ident_re = re.compile('^[a-zA-Z\u00a0-\U0010ffff_-][a-zA-Z0-9\u00a0-\U0010ffff_-]*$',
-                      flags=re.UNICODE)
+ident_re = re.compile(
+    '^[a-zA-Z\u00a0-\U0010ffff_-][a-zA-Z0-9\u00a0-\U0010ffff_-]*$', flags=re.UNICODE
+)
 initial_re = re.compile('^[^a-zA-Z_-]')
 subsequent_re = re.compile('[^a-zA-Z0-9_-]')
 trailing_digits_re = re.compile('_([0-9]+)$')
@@ -36,7 +37,7 @@ def isIdentifier(maxLength, obj):
 
 def forceIdentifier(maxLength, s):
     if not isinstance(s, str):
-        raise TypeError(f"{repr(str)} cannot be coerced to an identifier")
+        raise TypeError(f"{str!r} cannot be coerced to an identifier")
 
     # usually bytes2unicode can handle it
     s = util.bytes2unicode(s)
@@ -54,10 +55,10 @@ def incrementIdentifier(maxLength, ident):
     num = 1
     mo = trailing_digits_re.search(ident)
     if mo:
-        ident = ident[:mo.start(1) - 1]
+        ident = ident[: mo.start(1) - 1]
         num = int(mo.group(1))
     num = f'_{num + 1}'
     if len(num) > maxLength:
         raise ValueError("cannot generate a larger identifier")
-    ident = ident[:maxLength - len(num)] + num
+    ident = ident[: maxLength - len(num)] + num
     return ident
