@@ -12,14 +12,14 @@
 # Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 # Copyright Buildbot Team Members
-
+from __future__ import annotations
 
 from fnmatch import fnmatch
 
 
 class PullRequestMixin:
-    external_property_whitelist = []
-    external_property_denylist = []
+    external_property_whitelist: list[str] = []
+    external_property_denylist: list[str] = []
 
     def extractProperties(self, payload):
         def flatten(properties, base, info_dict):
@@ -29,8 +29,7 @@ class PullRequestMixin:
                     continue
                 if isinstance(v, dict):
                     flatten(properties, name, v)
-                elif any([fnmatch(name, expr)
-                          for expr in self.external_property_whitelist]):
+                elif any(fnmatch(name, expr) for expr in self.external_property_whitelist):
                     properties[name] = v
 
         properties = {}
