@@ -32,6 +32,7 @@ from buildbot.process.properties import Properties
 from buildbot.util import bytes2unicode
 from buildbot.util import config
 from buildbot.util import unicode2bytes
+from buildbot.warnings import warn_deprecated
 
 
 class ReconfigurableServiceMixin:
@@ -54,8 +55,6 @@ class ReconfigurableServiceMixin:
             yield svc.reconfigServiceWithBuildbotConfig(new_config)
 
 
-# twisted 16's Service is now an new style class, better put everybody new style
-# to catch issues even on twisted < 16
 class AsyncService(service.Service):
     name: str | None  # type: ignore[assignment]
 
@@ -131,6 +130,20 @@ class AsyncMultiService(AsyncService, service.MultiService):
         return defer.succeed(None)
 
 
+class IndependentAsyncMultiService(AsyncMultiService):
+    # Used in cases where lifetime must be managed differently than the parent
+
+    name: str | None  # type: ignore[assignment]
+    _master: None
+
+    def set_master(self, master):
+        self._master = master
+
+    @property
+    def master(self):
+        return self._master
+
+
 class MasterService(AsyncMultiService):
     # master service is the service that stops the master property recursion
 
@@ -138,10 +151,11 @@ class MasterService(AsyncMultiService):
     def master(self):
         return self
 
-    def get_db_url(self, new_config) -> defer.Deferred:
+    def get_db_config(self, new_config) -> defer.Deferred:
         p = Properties()
         p.master = self
-        return p.render(new_config.db['db_url'])
+
+        return p.render(new_config.db)
 
 
 class SharedService(AsyncMultiService):
@@ -537,6 +551,12 @@ class BuildbotServiceManager(AsyncMultiService, config.ConfiguredMixin, Reconfig
                 # so we implement switch of child when the service raises NotImplementedError
                 # Note this means that self will stop, and sibling will take ownership
                 # means that we have a small time where the service is unavailable.
+                warn_deprecated(
+                    '4.3.0',
+                    'raising NotImplementedError from '
+                    'reconfigServiceWithSibling() or reconfigService() has been deprecated',
+                )
+
                 yield svc.disownServiceParent()
                 config_sibling.objectid = svc.objectid
                 yield config_sibling.setServiceParent(self)
