@@ -13,6 +13,8 @@
 #
 # Copyright Buildbot Team Members
 
+from __future__ import annotations
+
 import sys
 
 from twisted.web import static
@@ -24,11 +26,11 @@ if sys.version_info[:2] >= (3, 9):
     # https://docs.python.org/3/library/importlib.resources.html
     import importlib.resources as importlib_resources
 else:
-    import importlib_resources
+    import importlib_resources  # type: ignore[import-not-found]
 
 
 class Application:
-    def __init__(self, package_name, description, ui=True):
+    def __init__(self, package_name: str, description: str, ui: bool = True) -> None:
         self.description = description
         self.version = importlib_resources.files(package_name).joinpath("VERSION")
         self.version = bytes2unicode(self.version.read_bytes())
@@ -36,15 +38,15 @@ class Application:
         self.resource = static.File(self.static_dir)
         self.ui = ui
 
-    def setMaster(self, master):
+    def setMaster(self, master: object) -> None:
         self.master = master
 
-    def setConfiguration(self, config):
+    def setConfiguration(self, config: object) -> None:
         self.config = config
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return (
-            "www.plugin.Application(version=%(version)s, "
-            "description=%(description)s, "
-            "static_dir=%(static_dir)s)"
-        ) % self.__dict__
+            "www.plugin.Application(version={version}, "
+            "description={description}, "
+            "static_dir={static_dir})"
+        ).format(**self.__dict__)
