@@ -13,6 +13,9 @@
 #
 # Copyright Buildbot Team Members
 
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
 
 from twisted.internet import defer
 
@@ -21,36 +24,39 @@ from buildbot.data import masters
 from buildbot.data import types
 from buildbot.db.schedulers import SchedulerAlreadyClaimedError
 
-
-class Db2DataMixin:
-    @defer.inlineCallbacks
-    def db2data(self, dbdict):
-        master = None
-        if dbdict['masterid'] is not None:
-            master = yield self.master.data.get(('masters', dbdict['masterid']))
-        data = {
-            'schedulerid': dbdict['id'],
-            'name': dbdict['name'],
-            'enabled': dbdict['enabled'],
-            'master': master,
-        }
-        return data
+if TYPE_CHECKING:
+    from buildbot.db.schedulers import SchedulerModel
+    from buildbot.util.twisted import InlineCallbacksType
 
 
-class SchedulerEndpoint(Db2DataMixin, base.Endpoint):
+@defer.inlineCallbacks
+def _db2data(master, dbdict: SchedulerModel):
+    dbmaster = None
+    if dbdict.masterid is not None:
+        dbmaster = yield master.data.get(('masters', dbdict.masterid))
+    data = {
+        'schedulerid': dbdict.id,
+        'name': dbdict.name,
+        'enabled': dbdict.enabled,
+        'master': dbmaster,
+    }
+    return data
+
+
+class SchedulerEndpoint(base.Endpoint):
     kind = base.EndpointKind.SINGLE
-    pathPatterns = """
-        /schedulers/n:schedulerid
-        /masters/n:masterid/schedulers/n:schedulerid
-    """
+    pathPatterns = [
+        "/schedulers/n:schedulerid",
+        "/masters/n:masterid/schedulers/n:schedulerid",
+    ]
 
     @defer.inlineCallbacks
     def get(self, resultSpec, kwargs):
         dbdict = yield self.master.db.schedulers.getScheduler(kwargs['schedulerid'])
         if 'masterid' in kwargs:
-            if dbdict['masterid'] != kwargs['masterid']:
+            if dbdict.masterid != kwargs['masterid']:
                 return None
-        return (yield self.db2data(dbdict)) if dbdict else None
+        return (yield _db2data(self.master, dbdict)) if dbdict else None
 
     @defer.inlineCallbacks
     def control(self, action, args, kwargs):
@@ -61,19 +67,19 @@ class SchedulerEndpoint(Db2DataMixin, base.Endpoint):
         return None
 
 
-class SchedulersEndpoint(Db2DataMixin, base.Endpoint):
+class SchedulersEndpoint(base.Endpoint):
     kind = base.EndpointKind.COLLECTION
-    pathPatterns = """
-        /schedulers
-        /masters/n:masterid/schedulers
-    """
+    pathPatterns = [
+        "/schedulers",
+        "/masters/n:masterid/schedulers",
+    ]
     rootLinkName = 'schedulers'
 
     @defer.inlineCallbacks
     def get(self, resultSpec, kwargs):
         schedulers = yield self.master.db.schedulers.getSchedulers(masterid=kwargs.get('masterid'))
         schdicts = yield defer.DeferredList(
-            [self.db2data(schdict) for schdict in schedulers],
+            [_db2data(self.master, schdict) for schdict in schedulers],
             consumeErrors=True,
             fireOnOneErrback=True,
         )
@@ -84,10 +90,9 @@ class Scheduler(base.ResourceType):
     name = "scheduler"
     plural = "schedulers"
     endpoints = [SchedulerEndpoint, SchedulersEndpoint]
-    keyField = 'schedulerid'
-    eventPathPatterns = """
-        /schedulers/:schedulerid
-    """
+    eventPathPatterns = [
+        "/schedulers/:schedulerid",
+    ]
 
     class EntityType(types.Entity):
         schedulerid = types.Integer()
@@ -95,7 +100,7 @@ class Scheduler(base.ResourceType):
         enabled = types.Boolean()
         master = types.NoneOk(masters.Master.entityType)
 
-    entityType = EntityType(name, 'Scheduler')
+    entityType = EntityType(name)
 
     @defer.inlineCallbacks
     def generateEvent(self, schedulerid, event):
@@ -104,35 +109,26 @@ class Scheduler(base.ResourceType):
 
     @base.updateMethod
     @defer.inlineCallbacks
-    def schedulerEnable(self, schedulerid, v):
+    def schedulerEnable(self, schedulerid: int, v: bool) -> InlineCallbacksType[None]:
         yield self.master.db.schedulers.enable(schedulerid, v)
         yield self.generateEvent(schedulerid, 'updated')
         return None
 
     @base.updateMethod
-    def findSchedulerId(self, name):
+    def findSchedulerId(self, name: str) -> defer.Deferred[int]:
         return self.master.db.schedulers.findSchedulerId(name)
 
     @base.updateMethod
-    def trySetSchedulerMaster(self, schedulerid, masterid):
-        d = self.master.db.schedulers.setSchedulerMaster(schedulerid, masterid)
-
-        # set is successful: deferred result is True
-        d.addCallback(lambda _: True)
-
-        @d.addErrback
-        def trapAlreadyClaimedError(why):
-            # the db layer throws an exception if the claim fails; we squash
-            # that error but let other exceptions continue upward
-            why.trap(SchedulerAlreadyClaimedError)
-
-            # set failed: deferred result is False
+    @defer.inlineCallbacks
+    def trySetSchedulerMaster(self, schedulerid: int, masterid: int) -> InlineCallbacksType[bool]:
+        try:
+            yield self.master.db.schedulers.setSchedulerMaster(schedulerid, masterid)
+        except SchedulerAlreadyClaimedError:
             return False
-
-        return d
+        return True
 
     @defer.inlineCallbacks
     def _masterDeactivated(self, masterid):
         schedulers = yield self.master.db.schedulers.getSchedulers(masterid=masterid)
         for sch in schedulers:
-            yield self.master.db.schedulers.setSchedulerMaster(sch['id'], None)
+            yield self.master.db.schedulers.setSchedulerMaster(sch.id, None)
