@@ -15,8 +15,11 @@
 
 from __future__ import annotations
 
+from copy import deepcopy
 from functools import reduce
 from typing import TYPE_CHECKING
+from typing import Any
+from typing import cast
 
 from twisted.internet import defer
 from twisted.internet import error
@@ -33,6 +36,7 @@ from buildbot.process.results import CANCELLED
 from buildbot.process.results import EXCEPTION
 from buildbot.process.results import FAILURE
 from buildbot.process.results import RETRY
+from buildbot.process.results import SKIPPED
 from buildbot.process.results import SUCCESS
 from buildbot.process.results import WARNINGS
 from buildbot.process.results import computeResultAndTermination
@@ -44,8 +48,13 @@ from buildbot.util import bytes2unicode
 from buildbot.util.eventual import eventually
 
 if TYPE_CHECKING:
+    from pathlib import PurePath
+
+    from buildbot.interfaces import IBuildStep
+    from buildbot.locks import BaseLockId
     from buildbot.process.builder import Builder
     from buildbot.process.workerforbuilder import AbstractWorkerForBuilder
+    from buildbot.util.subscription import Subscription
 
 
 class Build(properties.PropertiesMixin):
@@ -71,36 +80,44 @@ class Build(properties.PropertiesMixin):
     VIRTUAL_BUILDER_PROJECT_PROP = "virtual_builder_project"
     VIRTUAL_BUILDERTAGS_PROP = "virtual_builder_tags"
     workdir = "build"
+    workername: str | None
     reason = "changes"
     finished = False
-    results = None
+    results: int | None = None
     stopped = False
     set_runtime_properties = True
-    subs = None
+    subs: set[Subscription] | None = None
 
     class Sentinel:
         pass
 
     _sentinel = Sentinel()  # used as a sentinel to indicate unspecified initial_value
 
-    def __init__(self, requests):
+    def __init__(self, requests, builder: Builder) -> None:
         self.requests = requests
-        self.locks = []  # list of lock accesses
-        self._locks_to_acquire = []  # list of (real_lock, access) tuples
+        self.builder = builder
+        self.master = builder.master
+        self.workerforbuilder: AbstractWorkerForBuilder | None = None
+
+        self.locks: list[BaseLockId] = []  # list of lock accesses
+
+        self._locks_to_acquire: list[
+            tuple[BaseLockId, str]
+        ] = []  # list of (real_lock, access) tuples
         # build a source stamp
         self.sources = requests[0].mergeSourceStampsWith(requests[1:])
         self.reason = requests[0].mergeReasons(requests[1:])
 
-        self._preparation_step = None
-        self._locks_acquire_step = None
+        self._preparation_step: buildstep.BuildStep | None = None
+        self._locks_acquire_step: buildstep.BuildStep | None = None
         self.currentStep = None
 
-        self.workerEnvironment = {}
-        self.buildid = None
-        self._buildid_notifier = Notifier()
+        self.workerEnvironment: dict[str, str] = {}
+        self.buildid: int | None = None
+        self._buildid_notifier: Notifier[int] = Notifier()
         self.number = None
-        self.executedSteps = []
-        self.stepnames = {}
+        self.executedSteps: list[buildstep.BuildStep] = []
+        self.stepnames: dict[str, int] = {}
 
         self.terminate = False
 
@@ -119,21 +136,15 @@ class Build(properties.PropertiesMixin):
         self._is_substantiating = False
 
         # tracks the config version for locks
-        self.config_version = None
+        self.config_version = builder.config_version
+
+        self.env: dict[str, Any] = {}
+        if builder.config is not None:
+            self.env.update(deepcopy(builder.config.env))
 
     def getProperties(self):
         return self.properties
 
-    def setBuilder(self, builder: Builder) -> None:
-        """
-        Set the given builder as our builder.
-
-        @type  builder: L{buildbot.process.builder.Builder}
-        """
-        self.builder = builder
-        self.master = builder.master
-        self.config_version = builder.config_version
-
     def setLocks(self, lockList):
         self.locks = lockList
 
@@ -174,7 +185,7 @@ class Build(properties.PropertiesMixin):
 
     def __repr__(self):
         return (
-            f"<Build {self.builder.name} number:{repr(self.number)} "
+            f"<Build {self.builder.name} number:{self.number!r} "
             f"results:{statusToString(self.results)}>"
         )
 
@@ -214,7 +225,7 @@ class Build(properties.PropertiesMixin):
     def getWorkerCommandVersion(self, command, oldversion=None):
         return self.workerforbuilder.getWorkerCommandVersion(command, oldversion)
 
-    def getWorkerName(self):
+    def getWorkerName(self) -> str | None:
         return self.workername
 
     @staticmethod
@@ -264,21 +275,24 @@ class Build(properties.PropertiesMixin):
             props.setProperty("codebase", source.codebase, "Build")
             props.setProperty("project", source.project, "Build")
 
-    def setupWorkerBuildirProperty(self, workerforbuilder):
+    def setupWorkerProperties(self, workerforbuilder):
         path_module = workerforbuilder.worker.path_module
 
         # navigate our way back to the L{buildbot.worker.Worker}
         # object that came from the config, and get its properties
-        if workerforbuilder.worker.worker_basedir:
+        worker_basedir = workerforbuilder.worker.worker_basedir
+        if worker_basedir:
             builddir = path_module.join(
-                bytes2unicode(workerforbuilder.worker.worker_basedir),
+                bytes2unicode(worker_basedir),
                 bytes2unicode(self.builder.config.workerbuilddir),
             )
+            self.setProperty("basedir", worker_basedir, "Worker")
             self.setProperty("builddir", builddir, "Worker")
 
     def setupWorkerForBuilder(self, workerforbuilder: AbstractWorkerForBuilder):
         assert workerforbuilder.worker is not None
         self.path_module = workerforbuilder.worker.path_module
+        self.path_cls: type[PurePath] | None = workerforbuilder.worker.path_cls
         self.workername = workerforbuilder.worker.workername
         self.worker_info = workerforbuilder.worker.info
 
@@ -310,31 +324,40 @@ class Build(properties.PropertiesMixin):
         return self._builderid
 
     @defer.inlineCallbacks
-    def startBuild(self, workerforbuilder):
+    def startBuild(self, workerforbuilder: AbstractWorkerForBuilder):
         """This method sets up the build, then starts it by invoking the
         first Step. It returns a Deferred which will fire when the build
         finishes. This Deferred is guaranteed to never errback."""
         self.workerforbuilder = workerforbuilder
         self.conn = None
+        if self.builder.config is not None:
+            self.do_build = self.builder.config.do_build_if(self)
+        else:
+            self.do_build = True
 
         worker = workerforbuilder.worker
+        assert worker is not None
 
         # Cache the worker information as variables instead of accessing via worker, as the worker
         # will disappear during disconnection and some of these properties may still be needed.
         self.workername = worker.workername
         self.worker_info = worker.info
 
-        log.msg(f"{self}.startBuild")
+        log.msg(f"{self}.startBuild{' skipped' if not self.do_build else ''}")
 
         # TODO: this will go away when build collapsing is implemented; until
         # then we just assign the build to the first buildrequest
         brid = self.requests[0].id
         builderid = yield self.getBuilderId()
+        assert self.master is not None
+        assert self.master.data is not None
         self.buildid, self.number = yield self.master.data.updates.addBuild(
             builderid=builderid, buildrequestid=brid, workerid=worker.workerid
         )
+        assert self.buildid is not None
         self._buildid_notifier.notify(self.buildid)
 
+        assert self.master.mq is not None
         self.stopBuildConsumer = yield self.master.mq.startConsuming(
             self.controlStopBuild, ("control", "builds", str(self.buildid), "stop")
         )
@@ -347,14 +370,33 @@ class Build(properties.PropertiesMixin):
                 yield self.stopBuild(reason=reason)
                 return
 
+        if not self.do_build:
+            Build.setupBuildProperties(
+                self.getProperties(), self.requests, self.sources, self.number
+            )
+            metrics.MetricCountEvent.log('active_builds', 1)
+
+            # flush properties in the beginning of the build
+            yield self.master.data.updates.setBuildProperties(self.buildid, self)
+
+            self.results = SKIPPED
+            self.text: list[str] = []
+            yield self.allStepsDone()
+            return
+
         # the preparation step counts the time needed for preparing the worker and getting the
         # locks.
         # we cannot use a real step as we don't have a worker yet.
-        self._preparation_step = buildstep.create_step_from_step_or_factory(
-            buildstep.BuildStep(name="worker_preparation")
+        self._preparation_step = cast(
+            buildstep.BuildStep,
+            buildstep.create_step_from_step_or_factory(
+                buildstep.BuildStep(name="worker_preparation")
+            ),
         )
+        assert self._preparation_step is not None
         self._preparation_step.setBuild(self)
         yield self._preparation_step.addStep()
+        assert self.master.data.updates is not None
         yield self.master.data.updates.startStep(self._preparation_step.stepid, locks_acquired=True)
 
         Build.setupBuildProperties(self.getProperties(), self.requests, self.sources, self.number)
@@ -367,8 +409,11 @@ class Build(properties.PropertiesMixin):
             # not start builds that cannot acquire locks immediately. However on a loaded master
             # it may happen that more builds are cleared to start than there are free locks. In
             # such case some of the builds will be blocked and wait for the locks.
-            self._locks_acquire_step = buildstep.create_step_from_step_or_factory(
-                buildstep.BuildStep(name="locks_acquire")
+            self._locks_acquire_step = cast(
+                buildstep.BuildStep,
+                buildstep.create_step_from_step_or_factory(
+                    buildstep.BuildStep(name="locks_acquire")
+                ),
             )
             self._locks_acquire_step.setBuild(self)
             yield self._locks_acquire_step.addStep()
@@ -390,7 +435,7 @@ class Build(properties.PropertiesMixin):
         yield self.master.data.updates.setBuildProperties(self.buildid, self)
         yield self.master.data.updates.setBuildStateString(self.buildid, 'preparing worker')
         try:
-            ready_or_failure = False
+            ready_or_failure: bool | Failure = False
             if workerforbuilder.worker and workerforbuilder.worker.acquireLocks():
                 self._is_substantiating = True
                 ready_or_failure = yield workerforbuilder.substantiate_if_needed(self)
@@ -438,13 +483,14 @@ class Build(properties.PropertiesMixin):
         )
         yield self.master.data.updates.finishStep(self._preparation_step.stepid, SUCCESS, False)
 
+        assert workerforbuilder.worker is not None
         self.conn = workerforbuilder.worker.conn
 
-        # To retrieve the builddir property, the worker must be attached as we
-        # depend on its path_module. Latent workers become attached only after
-        # preparing them, so we can't setup the builddir property earlier like
-        # the rest of properties
-        self.setupWorkerBuildirProperty(workerforbuilder)
+        # To retrieve the worker properties, the worker must be attached as we depend on its
+        # path_module for at least the builddir property. Latent workers become attached only after
+        # preparing them, so we can't setup the builddir property earlier like the rest of
+        # properties
+        self.setupWorkerProperties(workerforbuilder)
         self.setupWorkerForBuilder(workerforbuilder)
         self.subs = self.conn.notifyOnDisconnect(self.lostRemote)
 
@@ -459,6 +505,7 @@ class Build(properties.PropertiesMixin):
         if self._locks_to_acquire:
             yield self.master.data.updates.setBuildStateString(self.buildid, "acquiring locks")
             locks_acquire_start_at = int(self.master.reactor.seconds())
+            assert self._locks_acquire_step is not None
             yield self.master.data.updates.startStep(
                 self._locks_acquire_step.stepid, started_at=locks_acquire_start_at
             )
@@ -524,18 +571,17 @@ class Build(properties.PropertiesMixin):
             lock.claim(self, access)
         return defer.succeed(None)
 
-    def setUniqueStepName(self, step):
+    def setUniqueStepName(self, name: str) -> str:
         # If there are any name collisions, we add a count to the loser
         # until it is unique.
-        name = step.name
         if name in self.stepnames:
             count = self.stepnames[name]
             count += 1
             self.stepnames[name] = count
-            name = f"{step.name}_{count}"
+            name = f"{name}_{count}"
         else:
             self.stepnames[name] = 0
-        step.name = name
+        return name
 
     def setupBuildSteps(self, step_factories):
         steps = []
@@ -570,37 +616,25 @@ class Build(properties.PropertiesMixin):
         # Add the new steps to the end.
         self.steps.extend(self.setupBuildSteps(step_factories))
 
-    def getNextStep(self):
+    def getNextStep(self) -> IBuildStep | None:
         """This method is called to obtain the next BuildStep for this build.
-        When it returns None (or raises a StopIteration exception), the build
-        is complete."""
-        if not self.steps:
-            return None
-        if not self.conn:
-            return None
-        if self.terminate or self.stopped:
-            # Run any remaining alwaysRun steps, and skip over the others
-            while True:
-                s = self.steps.pop(0)
-                if s.alwaysRun:
-                    return s
-                if not self.steps:
-                    return None
-        else:
-            return self.steps.pop(0)
+        When it returns None, the build is complete."""
+        while self.steps and self.conn:
+            s = self.steps.pop(0)
+            if not (self.terminate or self.stopped) or s.alwaysRun:
+                return s
+
+        return None
 
     def startNextStep(self):
-        try:
-            s = self.getNextStep()
-        except StopIteration:
-            s = None
-        if not s:
+        next_step = self.getNextStep()
+        if next_step is None:
             return self.allStepsDone()
-        self.executedSteps.append(s)
-        self.currentStep = s
+        self.executedSteps.append(next_step)
+        self.currentStep = next_step
 
         # the following function returns a deferred, but we don't wait for it
-        self._start_next_step_impl(s)
+        self._start_next_step_impl(next_step)
         return defer.succeed(None)
 
     @defer.inlineCallbacks
@@ -637,7 +671,7 @@ class Build(properties.PropertiesMixin):
         text = None
         if isinstance(results, tuple):
             results, text = results
-        assert isinstance(results, type(SUCCESS)), f"got {repr(results)}"
+        assert isinstance(results, type(SUCCESS)), f"got {results!r}"
         summary = yield step.getBuildResultSummary()
         if 'build' in summary:
             text = [summary['build']]
